<?php
/**
 * User Registration Frontend ListingTable List
 *
 * @version 1.0.0
 * @package  URFrontendListing/ListTable
 */

namespace  WPEverest\URFrontendListing\Admin;

use WP_Query;

if ( ! class_exists( 'UR_List_Table' ) ) {
    include_once dirname( UR_PLUGIN_FILE ) . '/includes/abstracts/abstract-ur-list-table.php';
}

/**
 * Frontend Listing table list class.
 */
class ListTable extends \UR_List_Table {

	/**
	 * Initialize the Frontend Listing table list.
	 */
	public function __construct() {
		$this->post_type       = 'ur_frontend_listings';
		$this->page            = 'user-registration-frontend-list';
		$this->per_page_option = 'user_registration_frontend_listing_per_page';
		$this->sort_by         = array(
			'title'  => array( 'title', false ),
			'author' => array( 'author', false ),
			'date'   => array( 'date', false ),
		);
		$this->bulk_actions    = $this->urfl_bulk_actions();
		parent::__construct(
			array(
				'singular' => 'frontend-listing',
				'plural'   => 'frontend-listings',
				'ajax'     => false,
			)
		);
	}

	/**
	 * No items found text.
	 */
	public function no_items() {
		_e( 'No User Lists found.', 'user-registration-frontend-listing' );
	}

	/**
	 * Get list columns.
	 *
	 * @return array
	 */
	public function get_columns() {
		return array(
			'cb'        => '<input type="checkbox" />',
			'title'     => __( 'Title', 'user-registration-frontend-listing' ),
			'shortcode' => __( 'Shortcode', 'user-registration-frontend-listing' ),
			'author'    => __( 'Author', 'user-registration-frontend-listing' ),
			'date'      => __( 'Date', 'user-registration-frontend-listing' ),
		);
	}

	/**
	 * Post Edit Link.
	 *
	 * @param  object $row
	 *
	 * @return string
	 */
	public function get_edit_links( $row ) {
		return admin_url( 'post.php?post=' . $row->ID . '&action=edit' );
	}

	/**
	 * Post Duplicate Link.
	 *
	 * @param  object $row
	 *
	 * @return string
	 */
	public function get_duplicate_link( $row ) {
		return admin_url( 'post.php?post=' . $row->ID . '&action=edit' );
	}

	/**
	 * Return title column.
	 *
	 * @param  object $frontend_listings Frontend List datas.
	 *
	 * @return string
	 */
	public function get_row_actions( $frontend_listings ) {

		$edit_link        = $this->get_edit_links( $frontend_listings );
		$post_status      = $frontend_listings->post_status;
		$post_type_object = get_post_type_object( $frontend_listings->post_type );
		// Get actions.
		$actions = array(
			'id' => sprintf( __( 'ID: %d', 'user-registration-frontend-listing' ), $frontend_listings->ID ),
		);

		if ( current_user_can( $post_type_object->cap->edit_post, $frontend_listings->ID ) && 'trash' !== $post_status ) {
			$actions['edit'] = '<a href="' . esc_url( $edit_link ) . '">' . __( 'Edit', 'user-registration-frontend-listing' ) . '</a>';
		}

		if ( current_user_can( $post_type_object->cap->delete_post, $frontend_listings->ID ) ) {
			if ( 'trash' == $post_status ) {
				$actions['untrash'] = '<a aria-label="' . esc_attr__( 'Restore this item from the Trash', 'user-registration-frontend-listing' ) . '" href="' . wp_nonce_url( admin_url( sprintf( $post_type_object->_edit_link . '&amp;action=untrash', $frontend_listings->ID ) ), 'untrash-post_' . $frontend_listings->ID ) . '">' . esc_html__( 'Restore', 'user-registration-frontend-listing' ) . '</a>';
			} elseif ( EMPTY_TRASH_DAYS ) {
				$actions['trash'] = '<a class="submitdelete" aria-label="' . esc_attr__( 'Move this item to the Trash', 'user-registration-frontend-listing' ) . '" href="' . get_delete_post_link( $frontend_listings->ID ) . '">' . esc_html__( 'Trash', 'user-registration-frontend-listing' ) . '</a>';
			}
			if ( 'trash' == $post_status || ! EMPTY_TRASH_DAYS ) {
				$actions['delete'] = '<a class="submitdelete" aria-label="' . esc_attr__( 'Delete this item permanently', 'user-registration-frontend-listing' ) . '" href="' . get_delete_post_link( $frontend_listings->ID, '', true ) . '">' . esc_html__( 'Delete permanently', 'user-registration-frontend-listing' ) . '</a>';
			}
		}
		return $actions;
	}

	/**
	 * Return shortcode column.
	 *
	 * @param  object $frontend_listings Frontend List datas.
	 *
	 * @return string
	 */
	public function column_shortcode( $frontend_listings ) {

		$shortcode = '[user_registration_frontend_list id="' . $frontend_listings->ID . '"]';

		echo sprintf( '<input type="text" onfocus="this.select();" readonly="readonly" value=\'%s\' class="widefat code"></span>', esc_attr( $shortcode ) );
		?>
		<button id="copy-shortcode" class="button ur-copy-shortcode " href="#" data-tip="<?php esc_attr_e( 'Copy Shortcode ! ', 'user-registration' ); ?>" data-copied="<?php esc_attr_e( 'Copied ! ', 'user-registration' ); ?>">
			<span class="dashicons dashicons-admin-page"></span>
		</button>
		<?php
	}

	/**
	 * Return author column.
	 *
	 * @param  object $frontend_listings Frontend List datas.
	 *
	 * @return string
	 */
	public function column_author( $frontend_listings ) {
		$user = get_user_by( 'id', $frontend_listings->post_author );

		if ( ! $user ) {
			return '<span class="na">&ndash;</span>';
		}

		$user_name = ! empty( $user->data->display_name ) ? $user->data->display_name : $user->data->user_login;

		if ( current_user_can( 'edit_user' ) ) {
			return '<a href="' . esc_url(
				add_query_arg(
					array(
						'user_id' => $user->ID,
					),
					admin_url( 'user-edit.php' )
				)
			) . '">' . esc_html( $user_name ) . '</a>';
		}

		return esc_html( $user_name );
	}


	/**
	 * Return created at date column.
	 *
	 * @param  object $frontend_listings Frontend List datas.
	 *
	 * @return string
	 */
	public function column_date( $frontend_listings ) {
		$post = get_post( $frontend_listings->ID );

		if ( ! $post ) {
			return;
		}

		$t_time = mysql2date(
			__( 'Y/m/d g:i:s A', 'user-registration-frontend-listing' ),
			$post->post_date,
			true
		);
		$m_time = $post->post_date;
		$time   = mysql2date( 'G', $post->post_date )
				- get_option( 'gmt_offset' ) * 3600;

		$time_diff = time() - $time;

		if ( $time_diff > 0 && $time_diff < 24 * 60 * 60 ) {
			$h_time = sprintf(
				__( '%s ago', 'user-registration-frontend-listing' ),
				human_time_diff( $time )
			);
		} else {
			$h_time = mysql2date( __( 'Y/m/d', 'user-registration-frontend-listing' ), $m_time );
		}

		return '<abbr title="' . $t_time . '">' . $h_time . '</abbr>';
	}

	/**
	 * Get bulk actions.
	 *
	 * @return array
	 */
	protected function urfl_bulk_actions() {
		if ( isset( $_GET['status'] ) && 'trash' == $_GET['status'] ) {
			return array(
				'untrash' => __( 'Restore', 'user-registration-frontend-listing' ),
				'delete'  => __( 'Delete permanently', 'user-registration-frontend-listing' ),
			);
		}

		return array(
			'trash' => __( 'Move to trash', 'user-registration-frontend-listing' ),
		);
	}

	/**
	 * Render the list table page, including header, notices, status filters and table.
	 */
	public function display_page() {
		$this->prepare_items();
		if ( ! isset( $_GET['add-new-frontend-listing'] ) ) { // phpcs:ignore Standard.Category.SniffName.ErrorCode: input var okay, CSRF ok.
			?>
				<div class="wrap">
					<h1 class="wp-heading-inline"><?php esc_html_e( 'User Registration Frontend Listings', 'user-registration-frontend-listing' ); ?></h1>
					<a href="<?php echo esc_url( admin_url( 'post-new.php?post_type=ur_frontend_listings' ) ); ?>" class="page-title-action"><?php esc_html_e( 'Add New', 'user-registration-frontend-listing' ); ?></a>
					<hr class="wp-header-end">
					<form id="frontend-listing-list" method="get">
						<input type="hidden" name="page" value="user-registration-frontend-list" />
					<?php
						$this->views();
						$this->search_box( __( 'Search Listings', 'user-registration-frontend-listing' ), 'frontend-listings' );
						$this->display();
					?>
					</form>
				</div>
				<?php
		}
	}
}
