<?php
/**
 * User_Registration_Frontend_Listing setup
 *
 * @package User_Registration_Frontend_Listing
 * @since  1.0.0
 */

namespace WPEverest\URFrontendListing;

use WPEverest\URFrontendListing\Admin\Admin;
use WPEverest\URFrontendListing\Frontend\Frontend;
use WPEverest\URFrontendListing\Admin\Shortcodes;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'FrontendListing' ) ) :

	/**
	 * Main FrontendListing Class
	 *
	 * @class FrontendListing
	 */
	final class FrontendListing {


		/**
		 * Instance of this class.
		 *
		 * @var object
		 */
		protected static $instance = null;

		/**
		 * Plugin Version
		 *
		 * @var string
		 */
		const VERSION = UR_FRONTEND_LISTING_VERSION;

		/**
		 * Admin class instance
		 *
		 * @var \Admin
		 * @since 1.0.0
		 */
		public $admin = null;

		/**
		 * Frontend class instance
		 *
		 * @var \Frontend
		 * @since 1.0.0
		 */
		public $frontend = null;

		/**
		 * Ajax.
		 *
		 * @since 1.0.0
		 *
		 * @var use WPEverest\URFrontendListing\AJAX;
		 */
		public $ajax = null;

		/**
		 * Shortcodes.
		 *
		 * @since 1.0.0
		 *
		 * @var WPEverest\URFrontendListing\Admin\Shortcodes;
		 */
		public $shortcodes = null;

		/**
		 * Return an instance of this class
		 *
		 * @return object A single instance of this class.
		 */
		public static function get_instance() {
			// If the single instance hasn't been set, set it now.
			if ( is_null( self::$instance ) ) {
				self::$instance = new self();
			}
			return self::$instance;
		}

		/**
		 * Constructor
		 *
		 * @since 1.0.0
		 */
		private function __construct() {
			require 'Functions/CoreFunctions.php';
			$message = ur_frontend_listing_is_compatible();

			// Checks if user registration plugin is installed.
			if ( 'YES' === $message ) {

				// Actions and Filters.
				add_filter( 'plugin_action_links_' . plugin_basename( UR_FRONTEND_LISTING_PLUGIN_FILE ), array( $this, 'plugin_action_links' ) );
				add_action( 'init', array( $this, 'create_post_type' ), 0 );
				add_action( 'init', array( $this, 'includes' ) );
				$this->load_plugin_textdomain();

			} else {
				ur_frontend_listing_check_plugin_compatibility();
			}
		}

		/**
		 * Plugin Updater.
		 */
		public function plugin_updater() {
			if ( function_exists( 'ur_addon_updater' ) ) {
				ur_addon_updater( UR_FRONTEND_LISTING_PLUGIN_FILE, 111978, self::VERSION );
			}
		}

		/**
		 * Includes.
		 */
		public function includes() {

			$this->ajax       = new Ajax();
			$this->shortcodes = new Shortcodes();

			// Class admin.
			if ( $this->is_admin() ) {
				// require file.
				$this->admin = new Admin();
			} else {
				// require file.
				$this->frontend = new Frontend();
			}
		}

		/**
		 * Check if is admin or not and load the correct class
		 *
		 * @return bool
		 * @since 1.0.0
		 */
		public function is_admin() {
			$check_ajax    = defined( 'DOING_AJAX' ) && DOING_AJAX;
			$check_context = isset( $_REQUEST['context'] ) && $_REQUEST['context'] == 'frontend';

			return is_admin() && ! ( $check_ajax && $check_context );
		}

		/**
		 * User Registration fallback notice.
		 */
		public function user_registration_missing_notice() {
			/* translators: %s: user-registration plugin link */
			echo '<div class="error notice is-dismissible"><p>' . sprintf( esc_html__( 'User Registration Frontend Listing requires %s version 1.9.5 or greater to work!', 'user-registration-frontend-listing' ), '<a href="https://wpeverest.com/wordpress-plugins/user-registration/" target="_blank">' . esc_html__( 'User Registration', 'user-registration-frontend-listing' ) . '</a>' ) . '</p></div>';
		}

		/**
		 * Display action links in the Plugins list table.
		 *
		 * @param array $actions Add plugin action link.
		 *
		 * @return array
		 */
		public function plugin_action_links( $actions ) {
			$new_actions = array(
				'settings' => '<a href="' . admin_url( 'admin.php?page=user-registration-frontend-list' ) . '" title="' . esc_attr( __( 'View User Registration Frontend Listing Settings', 'user-registration-frontend-listing' ) ) . '">' . __( 'Settings', 'user-registration-frontend-listing' ) . '</a>',
			);

			return array_merge( $new_actions, $actions );
		}

		/**
		 * Get the plugin url.
		 *
		 * @return string
		 */
		public function plugin_url() {
			return untrailingslashit( plugins_url( '/', __FILE__ ) );
		}

		/**
		 * Rgister Custom Post Type.
		 */
		public function create_post_type() {
			$raw_referer = wp_parse_args( wp_parse_url( wp_get_raw_referer(), PHP_URL_QUERY ) );
			$public      = isset( $_GET['add-new-frontend-listing'] ) || isset( $raw_referer['add-new-frontend-listing'] ) ? true : false;

			register_post_type(
				'ur_frontend_listings',
				apply_filters(
					'user_registration_frontend_listings_post_type',
					array(
						'labels'              => array(
							'name'                  => __( 'Listings', 'user-registration-frontend-listing' ),
							'singular_name'         => __( 'Listing', 'user-registration-frontend-listing' ),
							'all_items'             => __( 'All Listings', 'user-registration-frontend-listing' ),
							'menu_name'             => _x( 'Listings', 'Admin menu name', 'user-registration-frontend-listing' ),
							'add_new'               => __( 'Add New', 'user-registration-frontend-listing' ),
							'add_new_item'          => __( 'Add new', 'user-registration-frontend-listing' ),
							'edit'                  => __( 'Edit', 'user-registration-frontend-listing' ),
							'edit_item'             => __( 'Edit listing', 'user-registration-frontend-listing' ),
							'new_item'              => __( 'New listing', 'user-registration-frontend-listing' ),
							'view'                  => __( 'View listing', 'user-registration-frontend-listing' ),
							'view_item'             => __( 'View listings', 'user-registration-frontend-listing' ),
							'search_items'          => __( 'Search listings', 'user-registration-frontend-listing' ),
							'not_found'             => __( 'No listings found', 'user-registration-frontend-listing' ),
							'not_found_in_trash'    => __( 'No listings found in trash', 'user-registration-frontend-listing' ),
							'parent'                => __( 'Parent listing', 'user-registration-frontend-listing' ),
							'featured_image'        => __( 'Listing image', 'user-registration-frontend-listing' ),
							'set_featured_image'    => __( 'Set listing image', 'user-registration-frontend-listing' ),
							'remove_featured_image' => __( 'Remove listing image', 'user-registration-frontend-listing' ),
							'use_featured_image'    => __( 'Use as listing image', 'user-registration-frontend-listing' ),
							'insert_into_item'      => __( 'Insert into listing', 'user-registration-frontend-listing' ),
							'uploaded_to_this_item' => __( 'Uploaded to this listing', 'user-registration-frontend-listing' ),
							'filter_items_list'     => __( 'Filter listing', 'user-registration-frontend-listing' ),
							'items_list_navigation' => __( 'Listing navigation', 'user-registration-frontend-listing' ),
							'items_list'            => __( 'Listing list', 'user-registration-frontend-listing' ),

						),
						'public'              => $public,
						'show_ui'             => true,
						'capability_type'     => 'post',
						'map_meta_cap'        => true,
						'publicly_queryable'  => $public,
						'exclude_from_search' => $public,
						'show_in_menu'        => false,
						'hierarchical'        => false,
						'rewrite'             => false,
						'query_var'           => false,
						'supports'            => false,
						'show_in_nav_menus'   => false,
						'show_in_admin_bar'   => false,
						'supports'            => array( 'title' ),
					)
				)
			);

		}

		/**
		 * Load Localisation files.
		 *
		 * Note: the first-loaded translation file overrides any following ones if the same translation is present.
		 */
		public function load_plugin_textdomain() {
			$locale = is_admin() && function_exists( 'get_user_locale' ) ? get_user_locale() : get_locale();
			$locale = apply_filters( 'plugin_locale', $locale, 'user-registration-frontend-listing' );

			unload_textdomain( 'user-registration-frontend-listing' );

			load_textdomain( 'user-registration-frontend-listing', UR_FRONTEND_LISTING_DIR . 'languages/user-registration-frontend-listing-' . $locale . '.mo' );
			load_plugin_textdomain( 'user-registration-frontend-listing', false, UR_FRONTEND_LISTING_DIR . 'languages' );
		}

	}
endif;

/**
 * Main instance of FrontendListing.
 *
 * @since  1.0.0
 * @return FrontendListing
 */
function FrontendListing() {
	return FrontendListing::get_instance();
}
