<?php
/**
 * UserRegistrationTwoFactorAuthentication Ajax
 *
 * AJAX Event Handler
 *
 * @class    Ajax
 * @package  UserRegistrationTwoFactorAuthentication/Ajax
 * @category Ajax
 * @author   WPEverest
 * @since  1.0.0
 */

namespace WPEverest\UserRegistration\TwoFactorAuthentication;

use \Exception;
use \UR_Emailer;
use WPEverest\UserRegistration\TwoFactorAuthentication\OTP;
use \WPEverest\UserRegistration\TwoFactorAuthentication\Admin\Settings\Emails\UR_Settings_OTP_Email;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Ajax Class
 */
class Ajax {

	/**
	 * Hooks in ajax handlers
	 */
	public static function init() {
		add_action( 'wp_ajax_otp_submit', array( __class__, 'validate_otp' ) );
		add_action( 'wp_ajax_nopriv_otp_submit', array( __class__, 'validate_otp' ) );
		add_action( 'wp_ajax_otp_resend', array( __class__, 'resend_otp' ) );
		add_action( 'wp_ajax_nopriv_otp_resend', array( __class__, 'resend_otp' ) );
	}

	/**
	 * Send One Time Password email to user.
	 *
	 * @param [int]  $user_id User ID.
	 * @param string $otp OTP Code.
	 * @return void
	 */
	public static function send_otp_email( $user_id, $otp = '' ) {

		if ( 'PENDING' !== get_user_meta( $user_id, 'user_registration_tfa_status', true ) ) {
			return;
		}

		update_user_meta( $user_id, 'user_registration_tfa_otp_last_generated_on', time() );

		$user     = get_user_by( 'ID', $user_id );
		$username = $user->data->user_login;
		$email    = $user->data->user_email;

		$values = array(
			'username'        => $username,
			'email'           => $email,
			'otp_code'        => $otp,
			'otp_expiry_time' => get_option( 'user_registration_tfa_otp_expiry_time', 10 )
		);

		$header  = 'From: ' . UR_Emailer::ur_sender_name() . ' <' . UR_Emailer::ur_sender_email() . ">\r\n";
		$header .= 'Reply-To: ' . UR_Emailer::ur_sender_email() . "\r\n";
		$header .= "Content-Type: text/html\r\n; charset=UTF-8";

		$subject = get_option( 'user_registration_tfa_otp_email_subject', __( 'OTP Verification Code - {{blog_info}}', 'user-registration-two-factor-authentication' ) );

		$otp_email = new UR_Settings_OTP_Email();

		$message = $otp_email->get_otp_email();
		$message = get_option( 'user_registration_tfa_otp_email', $message );

		$message = UR_Emailer::parse_smart_tags( $message, $values );
		$subject = UR_Emailer::parse_smart_tags( $subject, $values );

		if ( 'yes' === get_option( 'user_registration_tfa_enable_otp_email', 'yes' ) ) {
			UR_Emailer::user_registration_process_and_send_email( $email, $subject, $message, $header, '', '' );
		}
	}

	/**
	 * Validate Submitted OTP.
	 *
	 * @return void
	 */
	public static function validate_otp() {

		$nonce = isset( $_REQUEST['security'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['security'] ) ) : '';

		if ( ! wp_verify_nonce( $nonce, 'otp_submit' ) ) {
			wp_send_json_error(
				array(
					'status'  => 'invalid_nonce',
					'message' => __( 'Security Error. Please try logging again.', 'user-registration-two-factor-authentication' )
				)
			);
		} else {
			$user_id = isset( $_POST['user_id'] ) ? sanitize_text_field( wp_unslash( $_POST['user_id'] ) ) : 0;

			if ( 'PENDING' !== get_user_meta( $user_id, 'user_registration_tfa_status', true ) ) {
				return;
			}

			$incorrect_attempts_left = (int) get_user_meta( $user_id, 'user_registration_tfa_incorrect_attempts_left', true );

			if ( $incorrect_attempts_left > 0 ) {

				update_user_meta( $user_id, 'user_registration_tfa_last_otp_submitted_on', time() );

				if ( self::is_otp_expired( $user_id ) ) {
					wp_send_json_error(
						array(
							'status'  => 'otp_expired',
							'message' => __( 'The OTP is expired. Please refresh this page to continue.', 'user-registration-two-factor-authentication' )
						)
					);
					exit;
				}

				$submitted_otp = isset( $_POST['otp_code'] ) ? sanitize_text_field( wp_unslash( $_POST['otp_code'] ) ) : '';
				$otp_matched   = OTP::verify_otp( $user_id, $submitted_otp );

				if ( $otp_matched ) {
					update_user_meta( $user_id, 'user_registration_tfa_status', 'VERIFIED' );

					$login_nonce = wp_create_nonce( 'user_registration_tfa_login_' . $user_id );

					$login_post    = get_post( get_option( 'user_registration_myaccount_page_id', 'unset' ) );
					$redirect_url  = home_url( $login_post->post_name . '/otp-login?' );
					$redirect_url .= 'uid=' . $user_id;
					$redirect_url .= '&_wpnonce=' . $login_nonce;

					wp_send_json_success(
						array(
							'status'   => 'verified',
							'message'  => __( 'OTP Matched. Redirecting...', 'user-registration-two-factor-authentication' ),
							'redirect' => $redirect_url
						)
					);
				} else {

					--$incorrect_attempts_left;

					if ( ! $incorrect_attempts_left ) {
						update_user_meta( $user_id, 'user_registration_tfa_status', 'HOLD' );
					}

					update_user_meta( $user_id, 'user_registration_tfa_incorrect_attempts_left', $incorrect_attempts_left );

					wp_send_json_error(
						array(
							'status'  => 'invalid_otp',
							'message' => get_option( 'user_registration_tfa_invalid_otp_message', __( 'Invalid OTP entered.', 'user-registration-two-factor-authentication' ) )
						)
					);
				}
			} else {
				wp_send_json_error(
					array(
						'status'  => 'otp_limit_reached',
						'message' => get_option( 'user_registration_tfa_invalid_otp_limit_message', __( 'Invalid OTP limit reached.', 'user-registration-two-factor-authentication' ) )
					)
				);
			}
		}
		exit;
	}


	/**
	 * Check if the generated OTP code has expired.
	 *
	 * @param [int] $user_id User ID.
	 * @return boolean
	 */
	public static function is_otp_expired( $user_id ) {
		$otp_expiry_time    = (int) get_option( 'user_registration_tfa_otp_expiry_time', true );
		$otp_generated_time = (int) get_user_meta( $user_id, 'user_registration_tfa_otp_last_generated_on', true );
		$otp_lifetime       = time() - $otp_generated_time;

		if ( $otp_lifetime > $otp_expiry_time * 60 ) {
			return true;
		}
		return false;
	}

	/**
	 * Generate new OTP and resend to user email.
	 *
	 * @return void
	 */
	public static function resend_otp() {
		$nonce = isset( $_REQUEST['security'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['security'] ) ) : '';
		if ( ! wp_verify_nonce( $nonce, 'otp_resend' ) ) {
			die( 'Invalid nonce.' );
		} else {
			$user_id = isset( $_POST['user_id'] ) ? sanitize_text_field( wp_unslash( $_POST['user_id'] ) ) : 0;

			if ( 'PENDING' !== get_user_meta( $user_id, 'user_registration_tfa_status', true ) ) {
				return;
			}

			$incorrect_attempts_left = (int) get_user_meta( $user_id, 'user_registration_tfa_incorrect_attempts_left', true );
			$resend_limit_left       = (int) get_user_meta( $user_id, 'user_registration_tfa_resend_attempts_left', true );

			if ( $incorrect_attempts_left ) {
				if ( $resend_limit_left ) {
					try {
						// 1. Generate One Time Password.
						$length   = get_option( 'user_registration_tfa_otp_length', 6 );
						$validity = get_option( 'user_registration_tfa_otp_expiry_time', 10 );
						$otp      = OTP::generate_otp( $user_id, $length, $validity );

						// 2. Send One Time Password to user email.
						self::send_otp_email( $user_id, $otp );

						--$resend_limit_left;

						update_user_meta( $user_id, 'user_registration_tfa_resend_attempts_left', $resend_limit_left );

						wp_send_json_success(
							array(
								'status'      => 'resent',
								'resent_left' => $resend_limit_left,
								'message'     => get_option( 'user_registration_tfa_otp_resent_message', __( 'OTP Sent successfully.', 'user-registration-two-factor-authentication' ) )
							)
						);

						exit;
					} catch ( Exception $e ) {
						wp_send_json_error(
							array(
								'status'      => 'error',
								'resent_left' => $resend_limit_left,
								'message'     => $e->getMessage()
							)
						);
					}
				} else {
					wp_send_json_error(
						array(
							'status'  => 'resend_limit_reached',
							'message' => get_option( 'user_registration_tfa_resend_limit_message', __( 'Resend OTP Limit reached.', 'user-registration-two-factor-authentication' ) )
						)
					);
				}
			} else {
				wp_send_json_error(
					array(
						'status'  => 'otp_limit_reached',
						'message' => get_option( 'user_registration_tfa_invalid_otp_limit_message', __( 'Invalid OTP Limit reached.', 'user-registration-two-factor-authentication' ) )
					)
				);
			}

			exit;
		}
	}
}

Ajax::init();
