<?php
/**
 * UserRegistrationTwoFactorAuthentication Frontend.
 *
 * @class    Frontend
 * @package  UserRegistrationTwoFactorAuthentication/Frontend
 * @category Admin
 * @author   WPEverest
 * @since  1.0.0
 */

namespace WPEverest\UserRegistration\TwoFactorAuthentication;

use \UR_Emailer;
use WPEverest\UserRegistration\TwoFactorAuthentication\OTP;
use WPEverest\UserRegistration\TwoFactorAuthentication\Ajax;

defined( 'ABSPATH' ) || exit;

/**
 * Frontend Class
 */
class Frontend {

	/**
	 * Hook in tabs.
	 */
	public function __construct() {

		add_filter( 'user_registration_get_query_vars', array( $this, 'add_otp_endpoint' ), 10, 1 );
		add_filter( 'authenticate', array( $this, 'start_two_factor_authentication' ), 50000, 3 );

		add_filter( 'user_registration_my_account_render_default', array( $this, 'prevent_default_render' ), 10, 2 );
		add_action( 'user_registration_my_account_custom_render', array( $this, 'render_otp_section' ) );

		add_action( 'wp_enqueue_scripts', array( $this, 'add_scripts' ) );
		add_action( 'user_registration_my_account_enqueue_scripts', array( $this, 'add_login_scripts' ), 11 );
		add_action( 'template_redirect', array( $this, 'login_user' ) );
		add_filter( 'wp_authenticate_user', array( $this, 'check_login_hold_status' ), 9, 2 );
	}

	/**
	 * Check if the user is on HOLD period and show error message.
	 *
	 * @param [object] $user WP_User || WP_Error.
	 * @return object WP_User || WP_Error
	 */
	public function check_login_hold_status( $user ) {
		if ( is_wp_error( $user ) ) {
			return $user;
		}

		if ( ur2fa_is_user_on_hold( $user->ID ) ) {
			$error_message = ur2fa_get_otp_limit_message( $user );

			return new \WP_Error( 'user_registration_error_message', $error_message );
		}

		return $user;
	}


	/**
	 * This is the main entry point for two factor authentication. Here, we check
	 * if two factor authentication is enabled and start the second factor
	 * authentication if the role matches.
	 *
	 * We add user metas, generate otp, send email and redirect to otp page.
	 *
	 * @param object $user WP_User cookie.
	 *
	 * @return string WP_User
	 */
	public function start_two_factor_authentication( $user, $username, $password ) {
		global $wp;

		if ( isset( $wp->query_vars['otp-login'] ) || is_wp_error( $user ) ) {
			return $user;
		}

		$two_factor_enabled = get_option( 'user_registration_tfa_enable_disable', 'no' );
		if ( 'yes' === $two_factor_enabled ) {

			$user_id = $user->ID;

			// Handling for Auto Approval and Auto Login.
			$current_time  = (int) strtotime( get_date_from_gmt( gmdate( 'Y-m-d H:i:s', time() ) ) );
			$user_lifetime = $current_time - (int) strtotime( $user->data->user_registered );

			if ( 5 > $user_lifetime ) {
				return $user;
			}

			// Only show otp verification to users registered from our forms.
			$form_id = ur_get_form_id_by_userid( $user_id );
			if ( 0 === $form_id ) {
				return $user;
			}

			$roles       = get_option( 'user_registration_tfa_roles', array() );
			$user_roles  = (array) $user->roles;
			$roles_match = array_intersect( $roles, $user_roles );

			if ( ! empty( $roles_match ) ) {
				$isset_ur2fa_meta = empty( get_user_meta( $user_id, 'user_registration_tfa_status' ) ) ? false : true;
				$ur2fa_status     = $isset_ur2fa_meta ? get_user_meta( $user_id, 'user_registration_tfa_status', true ) : '';

				if ( ! $isset_ur2fa_meta || 'VERIFIED' === $ur2fa_status ) {
					// Case: First login attempt.
					$this->start_fresh_session( $user_id );
				} elseif ( 'HOLD' === $ur2fa_status ) {
					$current_time         = time();
					$last_submission_time = get_user_meta( $user_id, 'user_registration_tfa_last_otp_submitted_on', true );
					$time_difference      = $current_time - $last_submission_time;
					$login_hold_period    = get_option( 'user_registration_tfa_login_hold_period', 60 );
					$login_hold_period    = 60 * (int) $login_hold_period;

					if ( $time_difference > $login_hold_period ) {
						$this->start_fresh_session( $user_id );
					}
				}
				$this->redirect_to_otp_page( $user_id );
			}
		}

		return $user;
	}


	/**
	 * Start Fresh Two Factor Authentication Session.
	 *
	 * @param [int] $user_id User ID.
	 * @return void
	 */
	public function start_fresh_session( $user_id ) {
		$this->delete_metas( $user_id );
		$this->set_init_metas( $user_id );
		$this->generate_and_send_otp( $user_id );
	}


	/**
	 * Prevent rendering of default login form to show otp form.
	 *
	 * @param [bool]  $default Default value.
	 * @param [array] $atts Attributes.
	 * @return bool
	 */
	public function prevent_default_render( $default, $atts ) {
		// When user is not logged in.
		global $wp;

		if ( ! isset( $wp->query_vars['otp'] ) || 'yes' !== get_option( 'user_registration_tfa_enable_disable', 'no' ) ) {
			return true;
		}

		$user_id = $this->get_user_id();

		$ur2fa_status = get_user_meta( $user_id, 'user_registration_tfa_status', true );

		if ( 'PENDING' === $ur2fa_status || 'HOLD' === $ur2fa_status ) {
			if ( $user_id <= 0 || ur_notice_count( 'error' ) ) {
				ur_print_notices();
				return true;
			}

			if ( Ajax::is_otp_expired( $user_id ) ) {
				$this->generate_and_send_otp( $user_id );
			}
			return false;
		}

		return true;
	}


	/**
	 * Render OTP template.
	 *
	 * @return void
	 */
	public function render_otp_section() {
		include UR2FA_ABSPATH . 'templates/otp-login.php';
	}


	/**
	 * Set user metas to initialize OTP verification.
	 *
	 * @param [int] $user_id User ID.
	 * @return void
	 */
	public function set_init_metas( $user_id ) {
		update_user_meta( $user_id, 'user_registration_tfa_status', 'PENDING' );

		$resend_limit = get_option( 'user_registration_tfa_otp_resend_limit', 3 );
		update_user_meta( $user_id, 'user_registration_tfa_resend_attempts_left', $resend_limit );

		$incorrect_limit = get_option( 'user_registration_tfa_incorrect_otp_limit', 5 );
		update_user_meta( $user_id, 'user_registration_tfa_incorrect_attempts_left', $incorrect_limit );
	}


	/**
	 * Delete metas set previously after user is logged in.
	 *
	 * @param [int] $user_id User Id.
	 * @return void
	 */
	public function delete_metas( $user_id ) {

		$metas = array(
			'user_registration_tfa_resend_attempts_left',
			'user_registration_tfa_incorrect_attempts_left',
			'user_registration_tfa_nonce',
			'user_registration_tfa_login_nonce',
			'user_registration_tfa_otp_key',
			'user_registration_tfa_otp_last_generated_on'
		);

		foreach ( $metas as  $meta ) {
			delete_user_meta( $user_id, $meta );
		}
	}


	/**
	 * Generate OTP and email it to user.
	 *
	 * @param [int] $user_id User id.
	 * @return void
	 */
	public function generate_and_send_otp( $user_id ) {
		// 1. Generate One Time Password.
		$length   = get_option( 'user_registration_tfa_otp_length', 6 );
		$validity = get_option( 'user_registration_tfa_otp_expiry_time', 10 );
		$otp      = OTP::generate_otp( $user_id, $length, $validity );

		// 2. Send One Time Password to user email.
		Ajax::send_otp_email( $user_id, $otp );
	}


	/**
	 * Validate and return current user id.
	 *
	 * @return int
	 */
	public function get_user_id() {
		$user_id = 0;

		if ( is_user_logged_in() ) {
			$user_id = get_current_user_id();
		} else {
			if ( isset( $_GET['uid'] ) ) {
				$user_id      = sanitize_text_field( wp_unslash( $_GET['uid'] ) );
				$nonce        = isset( $_GET['_wpnonce'] ) ? sanitize_text_field( wp_unslash( $_GET['_wpnonce'] ) ) : '';
				$nonce_action = 'user_registration_tfa_user_verification_' . $user_id;

				if ( ! wp_verify_nonce( $nonce, $nonce_action ) ) {
					ur_add_notice( __( 'Some error occured. Please login again.', 'user-registration-two-factor-authentication' ), 'error' );
					$user_id = -1;
				}
			} else {
				$user_id = 0;
				ur_add_notice( __( 'User not found. Please login again.', 'user-registration-two-factor-authentication' ), 'error' );
			}
		}
		return $user_id;
	}


	/**
	 * Set cookies and log in the user.
	 *
	 * @return void
	 */
	public function login_user() {
		global $wp;

		if ( ! isset( $wp->query_vars['otp-login'] ) ) {
			return;
		}

		$user_id = isset( $_GET['uid'] ) ? sanitize_text_field( wp_unslash( $_GET['uid'] ) ) : 0;

		if ( ! is_user_logged_in() && 0 !== $user_id ) {
			$login_nonce = isset( $_GET['_wpnonce'] ) ? sanitize_text_field( wp_unslash( $_GET['_wpnonce'] ) ) : '';

			if ( wp_verify_nonce( $login_nonce, 'user_registration_tfa_login_' . $user_id ) ) {
				$user = get_user_by( 'ID', $user_id );
				wp_set_current_user( $user_id, $user->user_login );
				wp_set_auth_cookie( $user_id );
				do_action( 'wp_login', $user->user_login, $user );

				$this->delete_metas( $user_id );
				$this->redirect_to_login_page();
			} else {
				ur_add_notice( __( 'Some error occured. Please login again.', 'user-registration-two-factor-authentication' ), 'error' );
			}
		} else {
			ur_add_notice( __( 'User not found. Please login again.', 'user-registration-two-factor-authentication' ), 'error' );
		}

		if ( ur_notice_count() > 0 ) {
			ur_print_notices();
			return true;
		}
	}


	/**
	 * Add necessary query vars to registered query vars.
	 *
	 * @param [array] $vars Registered query_vars.
	 * @return array $vars
	 */
	public function add_otp_endpoint( $vars ) {
		$vars['otp']       = 'otp';
		$vars['otp-login'] = 'otp-login';

		return $vars;
	}


	/**
	 * Add Scripts to handle error messages in login form.
	 *
	 * @return void
	 */
	public function add_login_scripts() {
		$suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

		wp_enqueue_script(
			'user-registration-two-factor-authentication-login',
			plugins_url( '/assets/js/two-factor-authentication-login' . $suffix . '.js', UR2FA_PLUGIN_FILE ),
			array( 'jquery' ),
			UR2FA_VERSION,
			true
		);
	}


	/**
	 * Register scripts and styles.
	 *
	 * @return void
	 */
	public function add_scripts() {
		global $wp;

		if ( ! isset( $wp->query_vars['otp'] ) ) {
			return;
		}

		$suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

		wp_enqueue_script(
			'user-registration-two-factor-authentication',
			plugins_url( '/assets/js/two-factor-authentication' . $suffix . '.js', UR2FA_PLUGIN_FILE ),
			array( 'jquery' ),
			UR2FA_VERSION,
			true
		);

		wp_enqueue_style(
			'user-registration-two-factor-authentication',
			plugins_url( '/assets/css/two-factor-authentication.css', UR2FA_PLUGIN_FILE ),
			array(),
			UR2FA_VERSION
		);

		$user_id = $this->get_user_id();

		if ( $user_id <= 0 ) {
			return;
		}

		$user = get_user_by( 'ID', $user_id );

		$values = array(
			'otp_length'               => get_option( 'user_registration_tfa_otp_length', 6 ),
			'otp_expiry_time'          => get_option( 'user_registration_tfa_otp_expiry_time', 10 ),
			'otp_resend_limit'         => get_user_meta( $user_id, 'user_registration_tfa_resend_attempts_left', true ),
			'otp_incorrect_limit'      => get_user_meta( $user_id, 'user_registration_tfa_incorrect_attempts_left', true ),
			'otp_sent_message'         => get_option( 'user_registration_tfa_otp_sent_message', '' ),
			'otp_resent_message'       => get_option( 'user_registration_tfa_otp_resent_message', '' ),
			'otp_invalid_message'      => get_option( 'user_registration_tfa_invalid_otp_message', '' ),
			'otp_resend_limit_message' => get_option( 'user_registration_tfa_resend_limit_message', '' ),
			'otp_submit_limit_message' => ur2fa_get_otp_limit_message( $user ),
			'ajax_url'                 => admin_url( 'admin-ajax.php' ),
			'otp_submit_action'        => esc_attr( 'otp_submit' ),
			'otp_submit_nonce'         => wp_create_nonce( 'otp_submit' ),
			'otp_resend_action'        => esc_attr( 'otp_resend' ),
			'otp_resend_nonce'         => wp_create_nonce( 'otp_resend' ),
			'login_hold_period'        => get_option( 'user_registration_tfa_login_hold_period', 60 ),
			'user_id'                  => $user_id,
			'login_page_url'           => ur_get_my_account_url()
		);

		wp_localize_script(
			'user-registration-two-factor-authentication',
			'user_registration_tfa_parameters',
			$values
		);
	}


	/**
	 * Redirect user to otp page.
	 *
	 * @param int $user_id User Id.
	 * @return void
	 */
	public function redirect_to_otp_page( $user_id = 0 ) {
		$nonce_action = 'user_registration_tfa_user_verification_' . $user_id;
		$nonce        = wp_create_nonce( $nonce_action );

		$redirect_url = add_query_arg(
			array(
				'uid'      => $user_id,
				'_wpnonce' => $nonce
			),
			ur_get_my_account_url() . '/otp'
		);

		if ( defined( 'DOING_AJAX' ) ) {
			wp_send_json_success(
				array(
					'message' => $redirect_url
				)
			);
		} else {
			wp_safe_redirect( $redirect_url );
		}
		exit;
	}


	/**
	 * Redirect user to login page.
	 *
	 * @return void
	 */
	public function redirect_to_login_page() {
		$redirect_url = ur_get_login_url();
		$redirect_url = ! empty( $redirect_url ) ? $redirect_url : ur_get_my_account_url();
		$redirect_url = ! empty( $redirect_url ) ? $redirect_url : get_home_url();

		wp_safe_redirect( $redirect_url );
		exit;
	}
}
