<?php
/**
 * UserRegistrationTwoFactorAuthentication Functions.
 *
 * General core functions available on both the front-end and admin.
 *
 * @author   WPEverest
 * @category Core
 * @package  UserRegistrationTwoFactorAuthentication/Functions
 * @version  1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
/**
 * Checks if Two Factor Authentication addon is compatible.
 *
 * @return string
 */
function ur2fa_is_compatible() {

	$ur_plugins_path     = WP_PLUGIN_DIR . UR2FA_DS . 'user-registration' . UR2FA_DS . 'user-registration.php';
	$ur_pro_plugins_path = WP_PLUGIN_DIR . UR2FA_DS . 'user-registration-pro' . UR2FA_DS . 'user-registration.php';

	if ( ! file_exists( $ur_plugins_path ) && ! file_exists( $ur_pro_plugins_path ) ) {
		return __( 'Please install <code>user-registration-pro</code> plugin to use <code>user-registration-two-factor-authentication</code> addon.', 'user-registration-two-factor-authentication' );
	}

	$ur_plugin_file_path     = 'user-registration/user-registration.php';
	$ur_pro_plugin_file_path = 'user-registration-pro/user-registration.php';

	include_once ABSPATH . 'wp-admin/includes/plugin.php';

	if ( ! is_plugin_active( $ur_plugin_file_path ) && ! is_plugin_active( $ur_pro_plugin_file_path ) ) {
		return __( 'Please activate <code>user-registration-pro</code> plugin to use <code>user-registration-two-factor-authentication</code> addon.', 'user-registration-two-factor-authentication' );
	}

	if ( function_exists( 'UR' ) ) {
		$user_registration_version = UR()->version;
	} else {
		$user_registration_version = get_option( 'user_registration_version' );
	}

	if ( ! is_plugin_active( $ur_pro_plugin_file_path ) ) {

		if ( version_compare( $user_registration_version, '2.2.3', '<' ) ) {
			return __( 'Please update your <code>user registration</code> plugin(to at least 2.2.3 version) to use <code>user-registration-two-factor-authentication</code> addon.', 'user-registration-two-factor-authentication' );
		}
	} else {

		if ( version_compare( $user_registration_version, '3.1.3', '<' ) ) {
			return __( 'Please update your <code>user-registration-pro</code> plugin(to at least 3.1.3 version) to use <code>user-registration-two-factor-authentication</code> addon.', 'user-registration-two-factor-authentication' );
		}
	}

	return 'YES';

}

/**
 * Checks Plugin compatibility
 */
function ur2fa_check_plugin_compatibility() {

	add_action( 'admin_notices', 'ur2fa_admin_notice' );

}

/**
 * Admin Notices.
 */
function ur2fa_admin_notice() {

	$class = 'notice notice-error';

	$message = ur2fa_is_compatible();

	if ( 'YES' !== $message ) {

		printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), $message );
	}
}

/**
 * Setting Page Contents for Two Factor Authentication.
 *
 * @return array
 */
function ur2fa_settings() {
	return apply_filters(
		'user_registration_tfa_settings',
		array(
			'title'    => __( 'Two Factor Authentication Settings', 'user-registration-two-factor-authentication' ),
			'sections' => array(
				'user_registration_tfa_general_settings' => array(
					'title'    => __( 'General', 'user-registration-two-factor-authentication' ),
					'type'     => 'card',
					'desc'     => '',
					'settings' => array(
						array(
							'title'    => __( 'Enable 2FA ?', 'user-registration-two-factor-authentication' ),
							'desc'     => __( 'Check To Enable Two Factor Authentication on user login.', 'user-registration-two-factor-authentication' ),
							'id'       => 'user_registration_tfa_enable_disable',
							'default'  => 'no',
							'type'     => 'toggle',
							'desc_tip' => true,
							'autoload' => false
						),

						array(
							'title'    => esc_html__( 'Roles to Enforce 2FA', 'user-registration-two-factor-authentication' ),
							'desc'     => __( 'Select user roles to enforce 2FA for users registered using User Registration forms.', 'user-registration-two-factor-authentication' ),
							'id'       => 'user_registration_tfa_roles',
							'default'  => array( array_search( 'subscriber', array_keys( ur_get_default_admin_roles() ) ) => 'subscriber' ),
							'type'     => 'multiselect',
							'class'    => 'ur-enhanced-select',
							'css'      => 'min-width: 350px;',
							'desc_tip' => true,
							'options'  => ur_get_default_admin_roles()
						)
					)
				),
				'user_registration_tfa_otp_settings' => array(
					'title'    => __( 'OTP Settings', 'user-registration-two-factor-authentication' ),
					'type'     => 'card',
					'desc'     => '',
					'settings' => array(
						array(
							'title'    => __( 'OTP Length', 'user-registration-two-factor-authentication' ),
							'desc'     => __( 'Choose the length of OTP code.', 'user-registration-two-factor-authentication' ),
							'desc_tip' => true,
							'id'       => 'user_registration_tfa_otp_length',
							'default'  => 6,
							'type'     => 'select',
							'class'    => 'ur-enhanced-select',
							'css'      => 'min-width: 350px;',
							'options'  => array(
								'4' => 4,
								'6' => 6,
								'8' => 8
							)
						),

						array(
							'title'             => __( 'OTP Expiry Time', 'user-registration-two-factor-authentication' ),
							'desc'              => __( 'Enter the time to expire the generated OTP ( in minutes ).', 'user-registration-two-factor-authentication' ),
							'desc_tip'          => true,
							'id'                => 'user_registration_tfa_otp_expiry_time',
							'default'           => 10,
							'type'              => 'number',
							'css'               => 'min-width: 350px;',
							'autoload'          => false,
							'class'             => 'user_registration_tfa_number_input',
							'custom_attributes' => array(
								'min' => 1
							)
						),

						array(
							'title'             => __( 'OTP Resend Limit', 'user-registration-two-factor-authentication' ),
							'desc'              => __( 'Enter the number of times to allow resending OTP.', 'user-registration-two-factor-authentication' ),
							'desc_tip'          => true,
							'id'                => 'user_registration_tfa_otp_resend_limit',
							'default'           => 3,
							'type'              => 'number',
							'css'               => 'min-width: 350px;',
							'autoload'          => false,
							'class'             => 'user_registration_tfa_number_input',
							'custom_attributes' => array(
								'min' => 1
							)
						),

						array(
							'title'             => __( 'Incorrect OTP Limit', 'user-registration-two-factor-authentication' ),
							'desc'              => __( 'Enter the number of times to allow incorrect OTP submission.', 'user-registration-two-factor-authentication' ),
							'desc_tip'          => true,
							'id'                => 'user_registration_tfa_incorrect_otp_limit',
							'default'           => 5,
							'type'              => 'number',
							'css'               => 'min-width: 350px;',
							'autoload'          => false,
							'class'             => 'user_registration_tfa_number_input',
							'custom_attributes' => array(
								'min' => 1
							)
						),

						array(
							'title'             => __( 'Login Hold Period', 'user-registration-two-factor-authentication' ),
							'desc'              => __( 'Enter the time to prevent user from logging in when user hits incorrect otp submission limit ( in minutes ).', 'user-registration-two-factor-authentication' ),
							'desc_tip'          => true,
							'id'                => 'user_registration_tfa_login_hold_period',
							'default'           => 60,
							'type'              => 'number',
							'css'               => 'min-width: 350px;',
							'autoload'          => false,
							'class'             => 'user_registration_tfa_number_input',
							'custom_attributes' => array(
								'min' => 1
							)
						)
					)
				),
				'user_registration_tfa_message_settings' => array(
					'title'    => __( 'OTP Messages', 'user-registration-two-factor-authentication' ),
					'type'     => 'card',
					'desc'     => '',
					'settings' => array(
						array(
							'title'       => __( 'OTP Sent Message', 'user-registration-two-factor-authentication' ),
							'desc'        => __( 'Enter the message to display when OTP has been sent to user email.', 'user-registration-two-factor-authentication' ),
							'id'          => 'user_registration_tfa_otp_sent_message',
							'placeholder' => '',
							'default'     => __( 'An email with a One Time Password(OTP) has been sent to your registered email address. Enter the OTP below to continue.', 'user-registration-two-factor-authentication' ),
							'type'        => 'textarea',
							'rows'        => 1,
							'cols'        => 40,
							'css'         => 'min-width: 350px; max-width: 350px; min-height: 100px;',
							'desc_tip'    => true
						),

						array(
							'title'       => __( 'OTP Resent Message', 'user-registration-two-factor-authentication' ),
							'desc'        => __( 'Enter the message to display when OTP is resent to user email.', 'user-registration-two-factor-authentication' ),
							'id'          => 'user_registration_tfa_otp_resent_message',
							'placeholder' => '',
							'default'     => __( 'An OTP has been resent. Please also check the spam folder. Enter the OTP below to continue.', 'user-registration-two-factor-authentication' ),
							'type'        => 'textarea',
							'rows'        => 1,
							'cols'        => 40,
							'css'         => 'min-width: 350px; max-width: 350px; min-height: 100px;',
							'desc_tip'    => true
						),

						array(
							'title'    => __( 'Invalid OTP Message', 'user-registration-two-factor-authentication' ),
							'desc'     => __( 'Enter the message to display when the entered OTP is incorrect.', 'user-registration-two-factor-authentication' ),
							'id'       => 'user_registration_tfa_invalid_otp_message',
							'default'  => __( 'The OTP you entered is invalid. Please try again.', 'user-registration-two-factor-authentication' ),
							'type'     => 'textarea',
							'rows'     => 1,
							'cols'     => 40,
							'css'      => 'min-width: 350px; max-width: 350px; min-height: 100px;',
							'desc_tip' => true
						),

						array(
							'title'    => __( 'Resend Limit Reached Message', 'user-registration-two-factor-authentication' ),
							'desc'     => __( 'Enter the message to display when the user reaches resend limit.', 'user-registration-two-factor-authentication' ),
							'id'       => 'user_registration_tfa_resend_limit_message',
							'default'  => __( 'You have reached the OTP resend limit.', 'user-registration-two-factor-authentication' ),
							'type'     => 'textarea',
							'rows'     => 1,
							'cols'     => 40,
							'css'      => 'min-width: 350px; max-width: 350px; min-height: 100px;',
							'desc_tip' => true
						),

						array(
							'title'    => __( 'Invalid Submission Limit Reached Message', 'user-registration-two-factor-authentication' ),
							'desc'     => __( 'Enter the message to show when the user reaches the invalid submission limit.', 'user-registration-two-factor-authentication' ),
							'id'       => 'user_registration_tfa_invalid_otp_limit_message',
							'default'  => __( 'You have reached the invalid submission limit. Please try to login again after {{otp_hold_time}}.', 'user-registration-two-factor-authentication' ),
							'type'     => 'textarea',
							'rows'     => 1,
							'cols'     => 40,
							'css'      => 'min-width: 350px; max-width: 350px; min-height: 100px;',
							'desc_tip' => true
						)
					)
				)
			)
		)
	);
}

if ( ! function_exists( 'ur2fa_is_user_on_hold' ) ) {
	/**
	 * Checks if a user is in HOLD state.
	 *
	 * @param [int] $user_id User ID.
	 * @return bool
	 */
	function ur2fa_is_user_on_hold( $user_id ) {
		$ur2fa_status = get_user_meta( $user_id, 'user_registration_tfa_status', true );
		if ( 'HOLD' === $ur2fa_status ) {
			$current_time         = time();
			$last_submission_time = get_user_meta( $user_id, 'user_registration_tfa_last_otp_submitted_on', true );
			$time_difference      = $current_time - $last_submission_time;
			$login_hold_period    = get_option( 'user_registration_tfa_login_hold_period', 60 );
			$login_hold_period    = 60 * (int) $login_hold_period;

			if ( $time_difference < $login_hold_period ) {
				return true;
			}
		}
		return false;
	}
}

if ( ! function_exists( 'ur2fa_get_otp_limit_message' ) ) {
	/**
	 * Get invalid otp submission limit reached error message.
	 *
	 * @param [User Object] $user User.
	 * @return string
	 */
	function ur2fa_get_otp_limit_message( $user ) {
		$error_message = get_option( 'user_registration_tfa_invalid_otp_limit_message', 'You have reached the invalid submission limit. Please try to login again after {{otp_hold_time}}.' );

		$remaining_hold_period = '';

		if ( ur2fa_is_user_on_hold( $user->ID ) ) {
			$remaining_hold_period = 'data-hold-period=' . ur2fa_get_remaining_hold_period( $user->ID );
		}

		$hold_time = sprintf(
			"<span id='user_registration_tfa_remaining_time' %s>some time</span>",
			esc_attr( $remaining_hold_period )
		);

		$error_message = UR_Emailer::parse_smart_tags(
			$error_message,
			array(
				'otp_hold_time' => $hold_time,
				'email'         => $user->data->user_email
			)
		);

		return $error_message;
	}
}


if ( ! function_exists( 'ur2fa_get_remaining_hold_period' ) ) {
	/**
	 * Returns the remaining hold period of user ( in minutes ).
	 *
	 * @param [int] $user_id User ID.
	 * @return int
	 */
	function ur2fa_get_remaining_hold_period( $user_id ) {
		if ( 0 >= intval( $user_id ) || ! ur2fa_is_user_on_hold( $user_id ) ) {
			return 0;
		}

		$last_submission_time = get_user_meta( $user_id, 'user_registration_tfa_last_otp_submitted_on', true );
		$login_hold_period    = get_option( 'user_registration_tfa_login_hold_period', 60 );
		$login_hold_period    = 60 * (int) $login_hold_period;

		$hold_end_time = $last_submission_time + $login_hold_period;
		$current_time  = time();

		$remaining_time = $hold_end_time - $current_time;
		$remaining_time = 0 < $remaining_time ? $remaining_time : 0;

		return intval( $remaining_time );
	}
}


if ( ! function_exists( 'ur_get_login_url' ) ) {
	/**
	 * Returns the full url of the login redirection page.
	 *
	 * @return string Complete Login Page address.
	 */
	function ur_get_login_url() {
		$page_id   = absint( get_option( 'user_registration_login_options_login_redirect_url', 'unset' ) );
		$permalink = 0 < $page_id ? get_permalink( $page_id ) : '';
		return $permalink;
	}
}

if ( ! function_exists( 'ur_get_my_account_url' ) ) {
	/**
	 * Returns the full url of the selected My Account page.
	 *
	 * If My Account Page is not set:
	 * 1. Checks if Prevent Core Login is enabled.
	 * 2. Returns Login Redirection Page url if set.
	 * 3. Else, returns default WordPress login url.
	 *
	 * @return string
	 */
	function ur_get_my_account_url() {
		$page_id   = absint( get_option( 'user_registration_myaccount_page_id', 'unset' ) );
		$permalink = 0 < $page_id ? get_permalink( $page_id ) : '';

		if ( $permalink ) {
			return $permalink;
		}

		$prevent_core_login = get_option( 'user_registration_login_options_prevent_core_login', 'no' );

		if ( 'yes' === $prevent_core_login ) {
			$login_redirect_page_id = get_option( 'user_registration_login_options_login_redirect_url', 'unset' );

			if ( 0 < $login_redirect_page_id ) {
				return get_permalink( $login_redirect_page_id );
			}
		}

		return wp_login_url();
	}
}
