<?php
/**
 * Plugin Name: User Registration Two Factor Authentication
 * Plugin URI: https://wpeverest.com/wordpress-plugins/user-registration/two-factor-authentication
 * Description: Two Factor Authentication addon for user registration plugin.
 * Version: 1.0.5
 * Author: WPEverest
 * UR requires at least: 2.2.3
 * UR tested up to: 2.3.3
 * Author URI: https://wpeverest.com
 * Text Domain: user-registration-two-factor-authentication
 * Domain Path: /languages/
 *
 * Copyright: © 2017 WPEverest.
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 *
 * @package UserRegistrationTwoFactorAuthentication
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if ( file_exists( dirname( __FILE__ ) . '/vendor/autoload.php' ) ) {
	require dirname( __FILE__ ) . '/vendor/autoload.php';
}

use \WPEverest\UserRegistration\TwoFactorAuthentication\Admin\Admin;
use \WPEverest\UserRegistration\TwoFactorAuthentication\Ajax;
use \WPEverest\UserRegistration\TwoFactorAuthentication\Frontend;
use \WPEverest\UserRegistration\TwoFactorAuthentication\OTP;

if ( ! class_exists( 'UserRegistrationTwoFactorAuthentication' ) ) :

	/**
	 * Main UserRegistrationTwoFactorAuthentication Class.
	 *
	 * @class   UserRegistrationTwoFactorAuthentication
	 * @version 1.0.1
	 */
	final class UserRegistrationTwoFactorAuthentication {

		/**
		 * Plugin version.
		 *
		 * @var string
		 */
		public $version = '1.0.5';


		/**
		 * Instance of this class.
		 *
		 * @var object
		 */
		protected static $_instance = null;

		/**
		 * OTP Class Instances
		 *
		 *  @var object
		 */
		public $otp = null;

		/**
		 * Forntend Class Instance
		 *
		 * @var object
		 */
		public $frontend = null;

		/**
		 * Admin class
		 *
		 * @var object
		 */
		public $admin = null;

		/**
		 * Return an instance of this class.
		 *
		 * @return object A single instance of this class.
		 */
		public static function instance() {
			// If the single instance hasn't been set, set it now.
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}

			return self::$_instance;
		}

		/**
		 * Cloning is forbidden.
		 *
		 * @since 1.0
		 */
		public function __clone() {
			_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'user-registration-two-factor-authentication' ), '1.0' );
		}

		/**
		 * Unserializing instances of this class is forbidden.
		 *
		 * @since 1.0
		 */
		public function __wakeup() {
			_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'user-registration-two-factor-authentication' ), '1.0' );
		}

		/**
		 * FlashToolkit Constructor.
		 */
		public function __construct() {
			$this->define_constants();

			require_once UR2FA_ABSPATH . 'includes/Functions.php';

			$message = ur2fa_is_compatible();
			if ( 'YES' !== $message ) {
				ur2fa_check_plugin_compatibility();
				return;
			}

			$this->includes();
			$this->init_hooks();

			do_action( 'user_registration_two_factor_authentication_loaded' );
		}

		/**
		 * Hook into actions and filters.
		 */
		private function init_hooks() {
			add_action( 'user_registration_loaded', array( $this, 'plugin_updater' ) );
			add_action( 'init', array( $this, 'init' ), 0 );

			register_activation_hook( __FILE__, array( $this, 'activation_actions' ) );
		}

		/**
		 * Plugin Updater.
		 */
		public function plugin_updater() {
			if ( function_exists( 'ur_addon_updater' ) ) {
				ur_addon_updater( __FILE__, 235111, $this->version );
			}
		}

		/**
		 * Define FT Constants.
		 */
		private function define_constants() {
			$this->define( 'UR2FA_DS', DIRECTORY_SEPARATOR );
			$this->define( 'UR2FA_PLUGIN_FILE', __FILE__ );
			$this->define( 'UR2FA_ABSPATH', dirname( __FILE__ ) . UR2FA_DS );
			$this->define( 'UR2FA_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );
			$this->define( 'UR2FA_VERSION', $this->version );

		}

		/**
		 * Define constant if not already set.
		 *
		 * @param string      $name Constant Name.
		 * @param string|bool $value Constant Value.
		 */
		private function define( $name, $value ) {
			if ( ! defined( $name ) ) {
				define( $name, $value );
			}
		}

		/**
		 * What type of request is this?
		 *
		 * @param  string $type admin, ajax, cron or frontend.
		 *
		 * @return bool
		 */
		private function is_request( $type ) {
			switch ( $type ) {
				case 'admin':
					return is_admin();
				case 'ajax':
					return defined( 'DOING_AJAX' );
				case 'cron':
					return defined( 'DOING_CRON' );
				case 'frontend':
					return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' );
			}
		}

		/**
		 * Includes.
		 */
		public function includes() {
			if ( $this->is_request( 'admin' ) ) {
				$this->admin = new Admin();
			}

			if ( $this->is_request( 'frontend' ) ) {
				$this->frontend = new Frontend();
			}

			$this->otp = new OTP();
			Ajax::init();
		}

		/**
		 * Init UserRegistrationTwoFactorAuthentication when WordPress Initialises.
		 */
		public function init() {
			// Before init action.
			do_action( 'before_user_registration_two_factor_authentication_init' );

			// Set up localisation.
			$this->load_plugin_textdomain();

			// Init action.
			do_action( 'user_registration_two_factor_authentication_init' );
		}

		/**
		 * Perform actions when plugin is activated.
		 */
		public function activation_actions() {
			add_rewrite_endpoint( 'otp-login', EP_ALL );
			add_rewrite_endpoint( 'otp', EP_ALL );
			flush_rewrite_rules();
		}

		/**
		 * Load Localisation files.
		 *
		 * Note: the first-loaded translation file overrides any following ones if the same translation is present.
		 *
		 * Locales found in:
		 *      - WP_LANG_DIR/user-registration-two-factor-authentication/user-registration-two-factor-authentication-LOCALE.mo
		 *      - WP_LANG_DIR/plugins/user-registration-two-factor-authentication-LOCALE.mo
		 */
		public function load_plugin_textdomain() {
			$locale = is_admin() && function_exists( 'get_user_locale' ) ? get_user_locale() : get_locale();
			$locale = apply_filters( 'plugin_locale', $locale, 'user-registration-two-factor-authentication' );

			unload_textdomain( 'user-registration-two-factor-authentication' );
			load_textdomain( 'user-registration-two-factor-authentication', WP_LANG_DIR . '/user-registration-two-factor-authentication/user-registration-two-factor-authentication-' . $locale . '.mo' );
			load_plugin_textdomain( 'user-registration-two-factor-authentication', false, plugin_basename( dirname( __FILE__ ) ) . '/languages' );
		}

		/**
		 * Get the plugin url.
		 *
		 * @return string
		 */
		public function plugin_url() {
			return untrailingslashit( plugins_url( '/', __FILE__ ) );
		}

		/**
		 * Get the plugin path.
		 *
		 * @return string
		 */
		public function plugin_path() {
			return untrailingslashit( plugin_dir_path( __FILE__ ) );
		}

		/**
		 * Get the template path.
		 *
		 * @return string
		 */
		public function template_path() {
			return apply_filters( 'user_registration_two_factor_authentication_template_path', 'user-registration-two-factor-authentication/' );
		}

		/**
		 * Get Ajax URL.
		 *
		 * @return string
		 */
		public function ajax_url() {
			return admin_url( 'admin-ajax.php', 'relative' );
		}
	}

endif;

/**
 * Main instance of UserRegistrationTwoFactorAuthentication.
 *
 * Returns the main instance of FT to prevent the need to use globals.
 *
 * @since  1.0.0
 * @return UserRegistrationTwoFactorAuthentication
 */
function URTFA() {
	return UserRegistrationTwoFactorAuthentication::instance();
}

// Global for backwards compatibility.
$GLOBALS['user-registration-two-factor-authentication'] = URTFA();
